/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 */
#ifndef __LIBAX25_UTIL_H
#define __LIBAX25_UTIL_H

#define unreachable() \
	do { __builtin_unreachable(); } while (0)

#endif /* __LIBAX25_UTIL_H  */
